// *=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*
// ** Copyright UCAR (c) 1992 - 2007
// ** University Corporation for Atmospheric Research (UCAR)
// ** National Center for Atmospheric Research (NCAR)
// ** Research Applications Lab (RAL)
// ** P.O.Box 3000, Boulder, Colorado, 80307-3000, USA
// *=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*

using namespace std;

///////////////////////////////////////////////////////////////////////////////

#include <cctype>
#include <iostream>
#include <sstream>
#include <unistd.h>
#include <cstdlib>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <cmath>

#include "vx_grib_classes/grib_classes.h"
#include "vx_grib_classes/grib_strings.h"

///////////////////////////////////////////////////////////////////////////////

extern void get_grib_code_list_str(int, int, int, char *);
extern void get_grib_level_list_str(int, int, int, char *);

///////////////////////////////////////////////////////////////////////////////

void get_grib_code_list_str(int k, int grib_code, int ncep_flag, char *str) {
   int i, match;

   for(i=0, match=-1; i<n_grib_code_list; i++) {

      if(grib_code == grib_code_list[i]) {
         match = i;
         break;
      }
   } // end for i

   if(match >= 0) {
      if(!ncep_flag &&
         grib_code_list[match] >= 128 &&
         grib_code_list[match] <= 254) {
         sprintf(str, "GC_%3i", grib_code_list[match]);
      }
      else {
         strcpy(str, grib_code_list_str[match][k]);
      }
   }
   else {
      strcpy(str, missing_str);
   }

   return;
}

///////////////////////////////////////////////////////////////////////////////

void get_grib_code_name(int grib_code, int ncep_flag, char *str) {

   get_grib_code_list_str(0, grib_code, ncep_flag, str);

   return;
}

///////////////////////////////////////////////////////////////////////////////

void get_grib_code_unit(int grib_code, int ncep_flag, char *str) {

   get_grib_code_list_str(1, grib_code, ncep_flag, str);

   return;
}

///////////////////////////////////////////////////////////////////////////////

void get_grib_code_abbr(int grib_code, int ncep_flag, char *str) {

   get_grib_code_list_str(2, grib_code, ncep_flag, str);

   return;
}

///////////////////////////////////////////////////////////////////////////////

void get_grib_level_list_str(int k, int grib_level, int ncep_flag, char *str) {
   int i, match;

   for(i=0, match=-1; i<n_grib_level_list; i++) {

      if(grib_level == grib_level_list[i]) {
         match = i;
         break;
      }
   } // end for i

   if(match >= 0) {
      if(!ncep_flag &&
         grib_level_list[match] >= 204 &&
         grib_level_list[match] <= 254) {
         sprintf(str, "GL_%3i", grib_level_list[match]);
      }
      else {
         strcpy(str, grib_level_list_str[match][k]);
      }
   }
   else {
      strcpy(str, missing_str);
   }

   return;
}

///////////////////////////////////////////////////////////////////////////////

void get_grib_level_name(int grib_level, int ncep_flag, char *str) {

   get_grib_level_list_str(0, grib_level, ncep_flag, str);

   return;
}

///////////////////////////////////////////////////////////////////////////////

void get_grib_level_abbr(int grib_level, int ncep_flag, char *str) {

   get_grib_level_list_str(1, grib_level, ncep_flag, str);

   return;
}

///////////////////////////////////////////////////////////////////////////////

void get_grib_level_str(int grib_level, int ncep_flag,
                       unsigned char *level_info, char *str) {
   int i, match;

   for(i=0, match=-1; i<n_grib_level_list; i++) {

      if(grib_level == grib_level_list[i]) {
         match = i;
         break;
      }
   } // end for i

   if(match < 0) {
      strcpy(str, missing_str);
   }
   else if(!ncep_flag &&
           grib_level >= 204 &&
           grib_level <= 254) {
      sprintf(str, "GL_%3i", grib_level);
   }
   else if(grib_level_flag[match] == 0) {
      strcpy(str, grib_level_list_str[match][1]);
   }
   else if(grib_level_flag[match] == 1) {
      sprintf(str, "%s_%i", grib_level_list_str[match][1], char2_to_int(level_info));
   }
   else if(grib_level_flag[match] == 2) {
      sprintf(str, "%s_%i_%i", grib_level_list_str[match][1],
              (int) level_info[0], (int) level_info[1]);
   }
   else {
      strcpy(str, missing_str);
   }

   return;
}

///////////////////////////////////////////////////////////////////////////////

int str_to_grib_code(const char *c) {
   int i, l, is_numeric, gc;
   char c_upper[512];

   gc = -1;

   // First check if this string contains a numeric grib code.
   // Check if each character in the string is a digit.
   is_numeric = 1;
   l = strlen(c);
   for(i=0; i<l; i++) {
      if(!isdigit(c[i])) {
         is_numeric = 0;
         break;
      }
   }

   if(is_numeric) {
      gc = atoi(c);
   }
   // If not numeric, search for a matching grib code abbreviation
   else{

      // Convert the input string to all upper case
      strcpy(c_upper, c);
      l = strlen(c_upper);
      for(i=0; i<l; i++) c_upper[i] = toupper(c_upper[i]);

      for(i=0; i<n_grib_code_list; i++) {
         if(strcmp(c_upper, grib_code_list_str[i][2]) == 0) {
            gc = grib_code_list[i];
            break;
         }
      }
   }

   // Check if a matching grib code was found
   if(gc == -1) {
      cerr << "\n\nERROR: str_to_grib_code() -> "
           << "can't find grib code corresponding to string \"" 
           << c << "\".\n\n" << flush;
      exit(1);
   }

   return(gc);
}

///////////////////////////////////////////////////////////////////////////////
