// *=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*
// ** Copyright UCAR (c) 1992 - 2007
// ** University Corporation for Atmospheric Research (UCAR)
// ** National Center for Atmospheric Research (NCAR)
// ** Research Applications Lab (RAL)
// ** P.O.Box 3000, Boulder, Colorado, 80307-3000, USA
// *=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*

////////////////////////////////////////////////////////////////////////

#ifndef GRIB_STRINGS_H
#define GRIB_STRINGS_H

////////////////////////////////////////////////////////////////////////

using namespace std;

////////////////////////////////////////////////////////////////////////

static const char missing_str[]  = "MISSING";

//
// Grib code for precipiation types
//
static const int prate_grib_code = 59;
static const int tstm_grib_code  = 60;
static const int apcp_grib_code  = 61;
static const int ncpcp_grib_code = 62;
static const int acpcp_grib_code = 63;

//
// Grib codes corresponding to P, Q, T, Z, U, V variables
//
static const int pres_grib_code = 1;  // Pressure
static const int spfh_grib_code = 51; // Specific Humidity
static const int tmp_grib_code  = 11; // Temperature
static const int hgt_grib_code  = 7;  // Geopotential Height
static const int ugrd_grib_code = 33; // U-component of wind
static const int vgrd_grib_code = 34; // V-component of wind

//
// Grib codes for quantities that can be derived from 
// the P, Q, T, Z, U, V variables
//
static const int dpt_grib_code   = 17; // Dewpoint temperature
static const int wdir_grib_code  = 31; // Wind direction
static const int wind_grib_code  = 32; // Wind speed
static const int rh_grib_code    = 52; // Relative humidity
static const int mixr_grib_code  = 53; // Humidity mixing ratio
static const int prmsl_grib_code = 2;  // Pressure Reduced to Mean Sea Level 

////////////////////////////////////////////////////////////////////////

//
// NCEP Office Note 388 - Table 2
// Grib Code to Units and Abbreviations
//
// Values for Grib Codes 128 to 254 are reserved for use by the 
// originating center but are set here to the NWS/NCEP usage
//
static const int n_grib_code_list = 256;
static const int grib_code_list[n_grib_code_list] = {
     0,   1,   2,   3,   4,   5,   6,   7,   8,   9,
    10,  11,  12,  13,  14,  15,  16,  17,  18,  19,
    20,  21,  22,  23,  24,  25,  26,  27,  28,  29,
    30,  31,  32,  33,  34,  35,  36,  37,  38,  39,
    40,  41,  42,  43,  44,  45,  46,  47,  48,  49,
    50,  51,  52,  53,  54,  55,  56,  57,  58,  59,
    60,  61,  62,  63,  64,  65,  66,  67,  68,  69,
    70,  71,  72,  73,  74,  75,  76,  77,  78,  79,
    80,  81,  82,  83,  84,  85,  86,  87,  88,  89,
    90,  91,  92,  93,  94,  95,  96,  97,  98,  99,
   100, 101, 102, 103, 104, 105, 106, 107, 108, 109,
   110, 111, 112, 113, 114, 115, 116, 117, 118, 119,
   120, 121, 122, 123, 124, 125, 126, 127, 128, 129,
   130, 131, 132, 133, 134, 135, 136, 137, 138, 139,
   140, 141, 142, 143, 144, 145, 146, 147, 148, 149,
   150, 151, 152, 153, 154, 155, 156, 157, 158, 159,
   160, 161, 162, 163, 164, 165, 166, 167, 168, 169,
   170, 171, 172, 173, 174, 175, 176, 177, 178, 179,
   180, 181, 182, 183, 184, 185, 186, 187, 188, 189,
   190, 191, 192, 193, 194, 195, 196, 197, 198, 199,
   200, 201, 202, 203, 204, 205, 206, 207, 208, 209,
   210, 211, 212, 213, 214, 215, 216, 217, 218, 219,
   220, 221, 222, 223, 224, 225, 226, 227, 228, 229,
   230, 231, 232, 233, 234, 235, 236, 237, 238, 239,
   240, 241, 242, 243, 244, 245, 246, 247, 248, 249,
   250, 251, 252, 253, 254, 255
};

//
// Grib Code Long Name, Grib Code Units, Grib Code Abbrev
//
static const char * const grib_code_list_str[n_grib_code_list][3] = {
   { "Reserved", "-", "-" },
   { "Pressure", "Pa", "PRES" },
   { "Pressure reduced to MSL", "Pa", "PRMSL" },
   { "Pressure tendency", "Pa/s", "PTEND" },
   { "Potential vorticity", "Km^2/kg/s", "PVORT" },
   { "ICAO Standard Atmosphere Reference Height", "m", "ICAHT" },
   { "Geopotential", "m^2/s^2", "GP" },
   { "Geopotential height", "gpm", "HGT" },
   { "Geometric height", "m", "DIST" },
   { "Standard deviation of height", "m", "HSTDV" },
   { "Total ozone", "Dobson", "TOZNE" },
   { "Temperature", "K", "TMP" },
   { "Virtual temperature", "K", "VTMP" },
   { "Potential temperature", "K", "POT" },
   { "Pseudo-adiabatic potential temperature or equivalent potential temperature", "K", "EPOT" },
   { "Maximum temperature", "K", "TMAX" },
   { "Minimum temperature", "K", "TMIN" },
   { "Dew point temperature", "K", "DPT" },
   { "Dew point depression (or deficit)", "K", "DEPR" },
   { "Lapse rate", "K/m", "LAPR" },
   { "Visibility", "m", "VIS" },
   { "Radar Spectra (1)", "-", "RDSP1" },
   { "Radar Spectra (2)", "-", "RDSP2" },
   { "Radar Spectra (3)", "-", "RDSP3" },
   { "Parcel lifted index (to 500 hPa)", "K", "PLI" },
   { "Temperature anomaly", "K", "TMPA" },
   { "Pressure anomaly", "Pa", "PRESA" },
   { "Geopotential height anomaly", "gpm", "GPA" },
   { "Wave Spectra (1)", "-", "WVSP1" },
   { "Wave Spectra (2)", "-", "WVSP2" },
   { "Wave Spectra (3)", "-", "WVSP3" },
   { "Wind direction (from which blowing)", "deg true", "WDIR" },
   { "Wind speed", "m/s", "WIND" },
   { "u-component of wind", "m/s", "UGRD" },
   { "v-component of wind", "m/s", "VGRD" },
   { "Stream function", "m^2/s", "STRM" },
   { "Velocity potential", "m^2/s", "VPOT" },
   { "Montgomery stream function", "m^2/s^2", "MNTSF" },
   { "Sigma coordinate vertical velocity", "/s", "SGCVV" },
   { "Vertical velocity (pressure)", "Pa/s", "VVEL" },
   { "Vertical velocity (geometric)", "m/s", "DZDT" },
   { "Absolute vorticity", "/s", "ABSV" },
   { "Absolute divergence", "/s", "ABSD" },
   { "Relative vorticity", "/s", "RELV" },
   { "Relative divergence", "/s", "RELD" },
   { "Vertical u-component shear", "/s", "VUCSH" },
   { "Vertical v-component shear", "/s", "VVCSH" },
   { "Direction of current", "Degree true", "DIRC" },
   { "Speed of current", "m/s", "SPC" },
   { "u-component of current", "m/s", "UOGRD" },
   { "v-component of current", "m/s", "VOGRD" },
   { "Specific humidity", "kg/kg", "SPFH" },
   { "Relative humidity", "%", "RH" },
   { "Humidity mixing ratio", "kg/kg", "MIXR" },
   { "Precipitable water", "kg/m^2", "PWAT" },
   { "Vapor pressure", "Pa", "VAPP" },
   { "Saturation deficit", "Pa", "SATD" },
   { "Evaporation", "kg/m^2", "EVP" },
   { "Cloud Ice", "kg/m^2", "CICE" },
   { "Precipitation rate", "kg/m^2/s", "PRATE" },
   { "Thunderstorm probability", "%", "TSTM" },
   { "Total precipitation", "kg/m^2", "APCP" },
   { "Large scale precipitation (non-conv.)", "kg/m^2", "NCPCP" },
   { "Convective precipitation", "kg/m^2", "ACPCP" },
   { "Snowfall rate water equivalent", "kg/m^2/s", "SRWEQ" },
   { "Water equiv. of accum. snow depth", "kg/m^2", "WEASD" },
   { "Snow depth", "m", "SNOD" },
   { "Mixed layer depth", "m", "MIXHT" },
   { "Transient thermocline depth", "m", "TTHDP" },
   { "Main thermocline depth", "m", "MTHD" },
   { "Main thermocline anomaly", "m", "MTHA" },
   { "Total cloud cover", "%", "TCDC" },
   { "Convective cloud cover", "%", "CDCON" },
   { "Low cloud cover", "%", "LCDC" },
   { "Medium cloud cover", "%", "MCDC" },
   { "High cloud cover", "%", "HCDC" },
   { "Cloud water", "kg/m^2", "CWAT" },
   { "Best lifted index (to 500 hPa)", "K", "BLI" },
   { "Convective snow", "kg/m^2", "SNOC" },
   { "Large scale snow", "kg/m^2", "SNOL" },
   { "Water Temperature", "K", "WTMP" },
   { "Land cover (land=1, sea=0) (see note)", "proportion", "LAND" },
   { "Deviation of sea level from mean", "m", "DSLM" },
   { "Surface roughness", "m", "SFCR" },
   { "Albedo", "%", "ALBDO" },
   { "Soil temperature", "K", "TSOIL" },
   { "Soil moisture content", "kg/m^2", "SOILM" },
   { "Vegetation", "%", "VEG" },
   { "Salinity", "kg/kg", "SALTY" },
   { "Density", "kg/m^3", "DEN" },
   { "Water runoff", "kg/m^2", "WATR" },
   { "Ice cover (ice=1, no ice=0) (See Note)", "proportion", "ICEC" },
   { "Ice thickness", "m", "ICETK" },
   { "Direction of ice drift", "deg. true", "DICED" },
   { "Speed of ice drift", "m/s", "SICED" },
   { "u-component of ice drift", "m/s", "UICE" },
   { "v-component of ice drift", "m/s", "VICE" },
   { "Ice growth rate", "m/s", "ICEG" },
   { "Ice divergence", "/s", "ICED" },
   { "Snow melt", "kg/m^2", "SNOM" },
   { "Significant height of combined wind waves and swell", "m", "HTSGW" },
   { "Direction of wind waves (from which)", "Degree true", "WVDIR" },
   { "Significant height of wind waves", "m", "WVHGT" },
   { "Mean period of wind waves", "s", "WVPER" },
   { "Direction of swell waves", "Degree true", "SWDIR" },
   { "Significant height of swell waves", "m", "SWELL" },
   { "Mean period of swell waves", "s", "SWPER" },
   { "Primary wave direction", "Degree true", "DIRPW" },
   { "Primary wave mean period", "s", "PERPW" },
   { "Secondary wave direction", "Degree true", "DIRSW" },
   { "Secondary wave mean period", "s", "PERSW" },
   { "Net short-wave radiation flux (surface)", "W/m^2", "NSWRS" },
   { "Net long wave radiation flux (surface)", "W/m^2", "NLWRS" },
   { "Net short-wave radiation flux (top of atmosphere)", "W/m^2", "NSWRT" },
   { "Net long wave radiation flux (top of atmosphere)", "W/m^2", "NLWRT" },
   { "Long wave radiation flux", "W/m^2", "LWAVR" },
   { "Short wave radiation flux", "W/m^2", "SWAVR" },
   { "Global radiation flux", "W/m^2", "GRAD" },
   { "Brightness temperature", "K", "BRTMP" },
   { "Radiance (with respect to wave number)", "W/m/sr", "LWRAD" },
   { "Radiance (with respect to wave length)", "W/m^3/sr", "SWRAD" },
   { "Latent heat net flux", "W/m^2", "LHTFL" },
   { "Sensible heat net flux", "W/m^2", "SHTFL" },
   { "Boundary layer dissipation", "W/m^2", "BLYDP" },
   { "Momentum flux, u component", "N/m^2", "UFLX" },
   { "Momentum flux, v component", "N/m^2", "VFLX" },
   { "Wind mixing energy", "J", "WMIXE" },
   { "Image data", "-", "IMGD" },
   { "Mean Sea Level Pressure (Standard Atmosphere Reduction)", "Pa", "MSLSA" },
   { "Mean Sea Level Pressure (MAPS System Reduction)", "Pa", "MSLMA" },
   { "Mean Sea Level Pressure (Eta Model Reduction)", "Pa", "MSLET" },
   { "Surface lifted index", "K", "LFTX" },
   { "Best (4 layer) lifted index", "K", "4LFTX" },
   { "K index", "K", "KX" },
   { "Sweat index", "K", "SX" },
   { "Horizontal moisture divergence", "kg/kg/s", "MCONV" },
   { "Vertical speed shear", "1/s", "VWSH" },
   { "3-hr pressure tendency Std. Atmos. Reduction", "Pa/s", "TSLSA" },
   { "Brunt-Vaisala frequency (squared)", "1/s^2", "BVF2" },
   { "Potential vorticity (density weighted)", "1/s/m", "PVMW" },
   { "Categorical rain (yes=1; no=0)", "non-dim", "CRAIN" },
   { "Categorical freezing rain (yes=1; no=0)", "non-dim", "CFRZR" },
   { "Categorical ice pellets (yes=1; no=0)", "non-dim", "CICEP" },
   { "Categorical snow (yes=1; no=0)", "non-dim", "CSNOW" },
   { "Volumetric soil moisture content", "fraction", "SOILW" },
   { "Potential evaporation rate", "W/m**2", "PEVPR" },
   { "Cloud workfunction", "J/kg", "CWORK" },
   { "Zonal flux of gravity wave stress", "N/m**2", "U-GWD" },
   { "Meridional flux of gravity wave stress", "N/m**2", "V-GWD" },
   { "Potential vorticity", "m**2/s/kg", "PV" },
   { "Covariance between meridional and zonal components of the wind. Defined as [uv]-[u][v], where \"[]\" indicates the mean over the indicated time span. ", "m^2/s^2", "COVMZ" },
   { "Covariance between temperature and zonal components of the wind. Defined as [uT]-[u][T], where \"[]\" indicates the mean over the indicated time span. ", "K*m/s", "COVTZ" },
   { "Covariance between temperature and meridional components of the wind. Defined as [vT]-[v][T], where \"[]\" indicates the mean over the indicated time span. ", "K*m/s", "COVTM" },
   { "Cloud water", "Kg/kg", "CLWMR" },
   { "Ozone mixing ratio", "Kg/kg", "O3MR" },
   { "Ground Heat Flux", "W/m^2", "GFLUX" },
   { "Convective inhibition", "J/kg", "CIN" },
   { "Convective Available Potential Energy", "J/kg", "CAPE" },
   { "Turbulent Kinetic Energy", "J/kg", "TKE" },
   { "Condensation pressure of parcel lifted from indicated surface", "Pa", "CONDP" },
   { "Clear Sky Upward Solar Flux", "W/m^2", "CSUSF" },
   { "Clear Sky Downward Solar Flux", "W/m^2", "CSDSF" },
   { "Clear Sky upward long wave flux", "W/m^2", "CSULF" },
   { "Clear Sky downward long wave flux", "W/m^2", "CSDLF" },
   { "Cloud forcing net solar flux", "W/m^2", "CFNSF" },
   { "Cloud forcing net long wave flux", "W/m^2", "CFNLF" },
   { "Visible beam downward solar flux", "W/m^2", "VBDSF" },
   { "Visible diffuse downward solar flux", "W/m^2", "VDDSF" },
   { "Near IR beam downward solar flux", "W/m^2", "NBDSF" },
   { "Near IR diffuse downward solar flux", "W/m^2", "NDDSF" },
   { "Rain water mixing ratio", "Kg/Kg", "RWMR" },
   { "Snow mixing ratio", "Kg/Kg", "SNMR" },
   { "Momentum flux", "N/m^2", "M FLX" },
   { "Mass point model surface", "non-dim", "LMH" },
   { "Velocity point model surface", "non-dim", "LMV" },
   { "Model layer number (from bottom up)", "non-dim", "MLYNO" },
   { "latitude (-90 to +90)", "deg", "NLAT" },
   { "east longitude (0-360)", "deg", "ELON" },
   { "Ice mixing ratio", "Kg/Kg", "ICMR" },
   { "Graupel mixing ratio", "Kg/Kg", "GRMR" },
   { "Surface wind gust", "m/s", "GUST" },
   { "x-gradient of log pressure", "1/m", "LPSX" },
   { "y-gradient of log pressure", "1/m", "LPSY" },
   { "x-gradient of height", "m/m", "HGTX" },
   { "y-gradient of height", "m/m", "HGTY" },
   { "Turbulence Potential Forecast Index", "non-dim", "TPFI" },
   { "Total Icing Potential Diagnostic", "non-dim", "TIPD" },
   { "Lightning", "non-dim", "LTNG" },
   { "Rate of water dropping from canopy to ground", "-", "RDRIP" },
   { "Virtual potential temperature", "K", "VPTMP" },
   { "Storm relative helicity", "m^2/s^2", "HLCY" },
   { "Probability from ensemble", "numeric", "PROB" },
   { "Probability from ensemble normalized with respect to climate expectancy", "numeric", "PROBN" },
   { "Probability of precipitation", "%", "POP" },
   { "Percent of frozen precipitation", "%", "CPOFP" },
   { "Probability of freezing precipitation", "%", "CPOZP" },
   { "u-component of storm motion", "m/s", "USTM" },
   { "v-component of storm motion", "m/s", "VSTM" },
   { "Number concentration for ice particles", " -", "NCIP" },
   { "Direct evaporation from bare soil", "W/m^2", "EVBS" },
   { "Canopy water evaporation", "W/m^2", "EVCW" },
   { "Ice-free water surface", "%", "ICWAT" },
   { "Convective weather detection index", "non-dim", "CWDI" },
   { "VAFTAD", "-", "VAFTD" },
   { "downward short wave rad. flux", "W/m^2", "DSWRF" },
   { "downward long wave rad. flux", "W/m^2", "DLWRF" },
   { "Ultra violet index (1 hour integration centered at solar noon)", "J/m^2", "UVI" },
   { "Moisture availability", "%", "MSTAV" },
   { "Exchange coefficient", "(kg/m^3)(m/s)", "SFEXC" },
   { "No. of mixed layers next to surface", "integer", "MIXLY" },
   { "Transpiration", "W/m^2", "TRANS" },
   { "upward short wave rad. flux", "W/m^2", "USWRF" },
   { "upward long wave rad. flux", "W/m^2", "ULWRF" },
   { "Amount of non-convective cloud", "%", "CDLYR" },
   { "Convective Precipitation rate", "kg/m^2/s", "CPRAT" },
   { "Temperature tendency by all physics", "K/s", "TTDIA" },
   { "Temperature tendency by all radiation", "K/s", "TTRAD" },
   { "Temperature tendency by non-radiation physics", "K/s", "TTPHY" },
   { "precip.index(0.0-1.00) (see note)", "fraction", "PREIX" },
   { "Std. dev. of IR T over 1x1 deg area", "K", "TSD1D" },
   { "Natural log of surface pressure", "ln(kPa)", "NLGSP" },
   { "Planetary boundary layer height", "m", "HPBL" },
   { "5-wave geopotential height", "gpm", "5WAVH" },
   { "Plant canopy surface water", "kg/m^2", "CNWAT" },
   { "Soil type (as in Zobler)", "Integer (0-9)", "SOTYP" },
   { "Vegitation type (as in SiB)", "Integer (0-13)", "VGTYP" },
   { "Blackadar's mixing length scale", "m", "BMIXL" },
   { "Asymptotic mixing length scale", "m", "AMIXL" },
   { "Potential evaporation", "kg/m^2", "PEVAP" },
   { "Snow phase-change heat flux", "W/m^2", "SNOHF" },
   { "5-wave geopotential height anomaly", "gpm", "5WAVA" },
   { "Convective cloud mass flux", "Pa/s", "MFLUX" },
   { "Downward total radiation flux", "W/m^2", "DTRF" },
   { "Upward total radiation flux", "W/m^2", "UTRF" },
   { "Baseflow-groundwater runoff", "kg/m^2", "BGRUN" },
   { "Storm surface runoff", "kg/m^2", "SSRUN" },
   { "Supercooled Large Droplet (SLD) Icing Potential Diagnostic", "non-dim", "SIPD" },
   { "Total ozone", "Kg/m^2", "03TOT" },
   { "Snow cover", "percent", "SNOWC" },
   { "Snow temperature", "K", "SNOT" },
   { "Covariance between temperature and vertical component of the wind. Defined as [wT]-[w][T], where \"[]\" indicates the mean over the indicated time span", "K*m/s", "COVTW" },
   { "Large scale condensat. heat rate", "K/s", "LRGHR" },
   { "Deep convective heating rate", "K/s", "CNVHR" },
   { "Deep convective moistening rate", "kg/kg/s", "CNVMR" },
   { "Shallow convective heating rate", "K/s", "SHAHR" },
   { "Shallow convective moistening rate", "kg/kg/s", "SHAMR" },
   { "Vertical diffusion heating rate", "K/s", "VDFHR" },
   { "Vertical diffusion zonal acceleration", "m/s^2", "VDFUA" },
   { "Vertical diffusion meridional accel", "m/s^2", "VDFVA" },
   { "Vertical diffusion moistening rate", "kg/kg/s", "VDFMR" },
   { "Solar radiative heating rate", "K/s", "SWHR" },
   { "long wave radiative heating rate", "K/s", "LWHR" },
   { "Drag coefficient", "non-dim", "CD" },
   { "Friction velocity", "m/s", "FRICV" },
   { "Richardson number", "non-dim.", "RI" },
   { "Missing", "-", "-" }
};

//
// NCEP Office Note 388 - Table 3
// Level Types and Values
//
static const int n_grib_level_list = 73;
static const int grib_level_list[n_grib_level_list] = {
     1,   2,   3,   4,   5,   6,   7,   8,   9,  20, 
   100, 101, 102, 103, 104, 105, 106, 107, 108, 109, 
   110, 111, 112, 113, 114, 115, 116, 117, 119, 120, 
   121, 125, 126, 128, 141, 160, 200, 201, 204, 206, 
   207, 209, 210, 211, 212, 213, 214, 215, 220, 222, 
   223, 224, 232, 233, 234, 235, 236, 237, 238, 239, 
   240, 242, 243, 244, 245, 246, 247, 248, 249, 251, 
   252, 253, 254
};

//
// Flags to indicate how to interpret the values of 
// Octets 11 and 12 in the PDS:
//    0 -> ignore them
//    1 -> combine them to form one number
//    2 -> octet 11 contains the beginning of a range and
//         octet 12 contains the end of a range
//
static const int grib_level_flag[n_grib_level_list] = {
     0,   0,   0,   0,   0,   0,   0,   0,   0,   2, 
     1,   2,   0,   1,   2,   1,   2,   1,   2,   1, 
     2,   1,   2,   1,   2,   1,   2,   1,   1,   2, 
     2,   1,   1,   2,   2,   1,   0,   0,   0,   0, 
     0,   0,   0,   0,   0,   0,   0,   0,   0,   0, 
     0,   0,   0,   0,   0,   0,   2,   0,   0,   0, 
     0,   0,   0,   0,   0,   0,   0,   0,   0,   0, 
     0,   0,   0
};

//
// Level Types and Values Long Name and Abbrev
//
static const char * const grib_level_list_str[n_grib_level_list][2] = {
   { "Ground or water surface", "SFC" },
   { "Cloud base level", "CBL" },
   { "Cloud top level", "CTL" },
   { "Level of 0 deg (C) isotherm", "0DEG" },
   { "Level of adiabatic condensation lifted from the surface", "ADCL" },
   { "Maximum wind level", "MWSL" },
   { "Tropopause", "TRO" },
   { "Nominal top of atmosphere", "NTAT" },
   { "Sea bottom", "SEAB" },
   { "Isothermal level (temperature in 1/100 K in octets 11 and 12)", "TMPL" },
   { "isobaric level pressure in hectoPascals (hPa) (2 octets)", "ISBL" },
   { "layer between two isobaric levels/pressure of top (kPa)/pressure of bottom (kPa)", "ISBY" },
   { "mean sea level", "MSL" },
   { "Specified altitude above MSL/altitude in meters", "GPML" },
   { "layer between two specified altitudes above MSL/altitude of top(hm)/altitude of bottom(hm)", "GPMY" },
   { "specified height level above ground/height in meters (2 octets)", "HTGL" },
   { "layer between two specified height levels above ground/height of top(hm)/height of bottom(hm)", "HTGY" },
   { "sigma level/sigma value in 1/10000 (2 octets)", "SIGL" },
   { "layer between two sigma levels/sigma value at top in 1/100/sigma value at bottom in 1/100", "SIGY" },
   { "Hybrid level/level number (2 octets)", "HYBL" },
   { "layer between two hybrid levels/level number at top/level number at bottom", "HYBY" },
   { "depth below land surface/centimeters (2 octets)", "DBLL" },
   { "layer between two depths below land surface/depth of upper surface(cm)/depth of lower surface(cm)", "DBLY" },
   { "isentropic (theta) level/Potential Temperature (K) (2 octets)", "THEL" },
   { "layer between two isentropic levels/475 K minus thEta of top in K/475 K minus thEta of bottom in K", "THEY" },
   { "level at specified pressure difference from ground to level/Pressure difference in hPa (2 octets)", "SPDL" },
   { "layer between two levels at specified pressure difference from ground to level/pressure difference from ground to top level in hPa/pressure difference from ground to bottom level in hPa", "SPDY" },
   { "potential vorticity(pv) surface/pv value in units of 10-9 Km^2/kgs (2 octets)", "PVL" },
   { "Eta level/Eta value in 1/10000 (2 octets)", "EtaL" },
   { "layer between two Eta levels/Eta value at top of layer in 1/100/Eta value at bottom of layer in 1/100", "EtaY" },
   { "layer between two isobaric surfaces (high precision)/1100 hPa minus pressure of top, in hPa/1100 hPa minus pressure of bottom, in hPa", "IBYH" },
   { "specified height level above ground (high precision)/Height in centimeters (2 octets)", "HGLH" },
   { "isobaric level/pressure in Pascals (Pa) (2 octets)", "ISBP" },
   { "layer between two sigma levels (high precision)/1.1 minus sigma of top, in 1/1000 of sigma/1.1 minus sigma of bottom, in 1/1000 of sigma", "SGYH" },
   { "layer between two isobaric surfaces (mixed precision)/pressure of top, in hPa/1100hPa minus pressure of bottom, in hPa", "IBYM" },
   { "depth below sea level/Depth in meters (2 octets)", "DBSL" },
   { "entire atmosphere (considered as a single layer) 0 (2 octets)", "EATM" },
   { "entire ocean (considered as a single layer) 0 (2 octets)", "EOCN" },
   { "Highest tropospheric freezing level", "HTFL" },
   { "Grid scale cloud bottom level", "GCBL" },
   { "Grid scale cloud top level", "GCTL" },
   { "Boundary layer cloud bottom level", "BCBL" },
   { "Boundary layer cloud top level", "BCTL" },
   { "Boundary layer cloud layer", "BCY" },
   { "Low cloud bottom level", "LCBL" },
   { "Low cloud top level", "LCTL" },
   { "Low cloud layer", "LCY" },
   { "Cloud ceiling", "CEIL" },
   { "Planetary Boundary Layer (derived from Richardson number)", "PBLRI" },
   { "Middle cloud bottom level", "MCBL" },
   { "Middle cloud top level", "MCTL" },
   { "Middle cloud layer", "MCY" },
   { "High cloud bottom level", "HCBL" },
   { "High cloud top level", "HCTL" },
   { "High cloud layer", "HCY" },
   { "Ocean Isotherm Level (1/10 deg C)", "OITL" },
   { "Layer between two depths below ocean surface/depth of upper surface (dam) (octet 11)/depth of lower surface (dam) (octet 12)", "OLYR" },
   { "Bottom of Ocean Mixed Layer (m)", "OBML" },
   { "Bottom of Ocean Isothermal Layer (m)", "OBIL" },
   { "Layer Ocean Surface and 26C Ocean Isothermal Level", "S26CY" },
   { "Ocean Mixed Layer", "OMXL" },
   { "Convective cloud bottom level", "CCBL" },
   { "Convective cloud top level", "CCTL" },
   { "Convective cloud layer", "CCY" },
   { "Lowest level of the wet bulb zero", "LLTW" },
   { "Maximum equivalent potential temperature level", "MTHE" },
   { "Equilibrium level", "EHLT" },
   { "Shallow convective cloud bottom level", "SCBL" },
   { "Shallow convective cloud top level", "SCTL" },
   { "Deep convective cloud bottom level", "DCBL" },
   { "Deep convective cloud top level", "DCTL" },
   { "Lowest bottom level of supercooled liquid water layer", "LBLSW" },
   { "Highest top level of supercooled liquid water layer", "HTLSW" }
};

////////////////////////////////////////////////////////////////////////

void get_grib_code_name(int, int, char *);
void get_grib_code_unit(int, int, char *);
void get_grib_code_abbr(int, int, char *);

void get_grib_level_name(int, int, char *);
void get_grib_level_abbr(int, int, char *);
void get_grib_level_str(int, int, unsigned char *, char *);

int str_to_grib_code(const char *);

////////////////////////////////////////////////////////////////////////

#endif   //  GRIB_STRINGS_H

////////////////////////////////////////////////////////////////////////

