// *=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*
// ** Copyright UCAR (c) 1992 - 2007
// ** University Corporation for Atmospheric Research (UCAR)
// ** National Center for Atmospheric Research (NCAR)
// ** Research Applications Lab (RAL)
// ** P.O.Box 3000, Boulder, Colorado, 80307-3000, USA
// *=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*



////////////////////////////////////////////////////////////////////////


#ifndef  __CONCAT_STRING_H__
#define  __CONCAT_STRING_H__


////////////////////////////////////////////////////////////////////////


#include <iostream>


////////////////////////////////////////////////////////////////////////


static const int concat_string_alloc_inc = 512;


////////////////////////////////////////////////////////////////////////


static const int concat_string_default_precision = 2;

static const int concat_string_max_precision = 12;


////////////////////////////////////////////////////////////////////////


class ConcatString {

   private:

      void init_from_scratch();

      void assign(const ConcatString &);

      void extend(int);


      int Length;

      int Nalloc;

      int Precision;

      char FloatFormat[16];

      char * s;

   public:

      ConcatString();
     ~ConcatString();
      ConcatString(const ConcatString &);
      ConcatString(const char *);
      ConcatString & operator=(const ConcatString &);
      ConcatString & operator=(const char *);

      void clear();

      void add(const char *);

      void add(const ConcatString &);

      void chomp();   //  removes trailing newline

      const char * text() const;

      int length() const;   //  not including trailing nul

      int precision() const;

      void set_precision(int);

      const char * float_format() const;

      operator const char * () const;

};


////////////////////////////////////////////////////////////////////////


inline const char * ConcatString::text() const { return ( s ); }

inline int ConcatString::length() const { return ( Length ); }

inline int ConcatString::precision() const { return ( Precision ); }

inline const char * ConcatString::float_format() const { return ( FloatFormat ); }

inline ConcatString::operator const char * () const { return ( s ); }


////////////////////////////////////////////////////////////////////////


extern ConcatString & operator<<(ConcatString &, const char);

extern ConcatString & operator<<(ConcatString &, const char *);

extern ConcatString & operator<<(ConcatString &, const ConcatString &);

extern ConcatString & operator<<(ConcatString &, int);

extern ConcatString & operator<<(ConcatString &, long long);

extern ConcatString & operator<<(ConcatString &, double);

extern ostream & operator<<(ostream &, const ConcatString &);


////////////////////////////////////////////////////////////////////////


#endif   //  __CONCAT_STRING_H__


////////////////////////////////////////////////////////////////////////


