// *=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*
// ** Copyright UCAR (c) 1992 - 2007
// ** University Corporation for Atmospheric Research (UCAR)
// ** National Center for Atmospheric Research (NCAR)
// ** Research Applications Lab (RAL)
// ** P.O.Box 3000, Boulder, Colorado, 80307-3000, USA
// *=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*


////////////////////////////////////////////////////////////////////////

#ifndef  __VX_WRFMODE_INTEREST_H__
#define  __VX_WRFMODE_INTEREST_H__

////////////////////////////////////////////////////////////////////////

#include <iostream>

#include "vx_wrfdata/vx_wrfdata.h"

////////////////////////////////////////////////////////////////////////

class SingleFeature {

   private:
      void assign(const SingleFeature &);

   public:
      SingleFeature();
     ~SingleFeature();
      SingleFeature(const SingleFeature &);
      SingleFeature & operator=(const SingleFeature &);

      void clear();

      void set(const WrfData &raw, const WrfData &mask, const double p);

      const WrfData *Raw;  //  NOT allocated, so DON'T delete!
      const WrfData *Mask; //  NOT allocated, so DON'T delete!

      int object_number;

      //
      // attributes
      //
      double area;
      double area_filter;

      double xbar;
      double ybar;

      double angle_degrees;
      double aspect_ratio;
      double complexity;
      double percentile_intensity;

      Polyline convex_hull;
      Polyline boundary;
};

////////////////////////////////////////////////////////////////////////

extern ostream & operator<<(ostream &, const SingleFeature &);

////////////////////////////////////////////////////////////////////////

class PairFeature {

   private:
      void assign(const PairFeature &);

   public:
      PairFeature();
     ~PairFeature();
      PairFeature(const PairFeature &);
      PairFeature & operator=(const PairFeature &);

      void clear();

      void set(const SingleFeature &fcst, const SingleFeature &obs, 
               double max_centroid_dist);

      const SingleFeature *Obs;    //  NOT allocated, so DON'T delete
      const SingleFeature *Fcst;   // 

      int pair_number;

      //
      // attributes
      //
      double centroid_dist;
      double boundary_dist;      
      double convex_hull_dist;
      double angle_diff;
      double area_ratio;
      double intersection_area;
      double union_area;
      double complexity_ratio;
      double percentile_intensity_ratio;
};

////////////////////////////////////////////////////////////////////////

extern ostream & operator<<(ostream &, const PairFeature &);

////////////////////////////////////////////////////////////////////////

#endif   //  __VX_WRFMODE_INTEREST_H__

////////////////////////////////////////////////////////////////////////
